@@ -31,16 +31,22 @@ unsigned _xklPhysIndicatorsMask;
 
 int _xklXkbEventType, _xklXkbError;
 
-Atom _xklAtoms[3];
+Atom _xklAtoms[TOTAL_ATOMS];
 
 Window _xklRootWindow;
 
 Bool _xklAllowSecondaryGroupOnce;
 
+int _xklDefaultGroup;
+
+Bool _xklSkipOneRestore;
+
 int _xklSecondaryGroupsMask;
 
 int _xklDebugLevel = 160;
 
+Window _xklPrevAppWindow;
+
 static XklConfigCallback configCallback = NULL;
 static void *configCallbackData;
 
@@ -90,9 +96,20 @@ Bool XklIsGroupPerApp( void )
 
 void XklAllowOneSwitchToSecondaryGroup( void )
 {
+  XklDebug( 150, "Setting allowOneSwitchToSecondaryGroup flag\n" );
   _xklAllowSecondaryGroupOnce = True;
 }
 
+void XklSetDefaultGroup( int group )
+{
+  _xklDefaultGroup = group;
+}
+
+int XklGetDefaultGroup( void )
+{
+  return _xklDefaultGroup;
+}
+
 void XklSetSecondaryGroupsMask( int mask )
 {
   _xklSecondaryGroupsMask = mask;
@@ -148,15 +165,25 @@ int XklInit( Display * a_dpy )
 
   scr = DefaultScreen( _xklDpy );
   _xklRootWindow = RootWindow( _xklDpy, scr );
-  XklDebug( 160, "xkbEvenType: %X, xkbError: %X, display: %p, root: %ld\n",
-            _xklXkbEventType, _xklXkbError, _xklDpy, _xklRootWindow );
+  XklDebug( 160,
+            "xkbEvenType: %X, xkbError: %X, display: %p, root: " WINID_FORMAT
+            "\n", _xklXkbEventType, _xklXkbError, _xklDpy, _xklRootWindow );
 
-  _xklAtoms[XKLAVIER_STATE] = XInternAtom( _xklDpy, "XKLAVIER_STATE", False );
-  _xklAtoms[WM_STATE] = XInternAtom( _xklDpy, "WM_STATE", False );
   _xklAtoms[WM_NAME] = XInternAtom( _xklDpy, "WM_NAME", False );
+  _xklAtoms[WM_STATE] = XInternAtom( _xklDpy, "WM_STATE", False );
+  _xklAtoms[XKLAVIER_STATE] = XInternAtom( _xklDpy, "XKLAVIER_STATE", False );
+  _xklAtoms[XKLAVIER_TRANSPARENT] =
+    XInternAtom( _xklDpy, "XKLAVIER_TRANSPARENT", False );
+  _xklAtoms[XKB_RF_NAMES_PROP_ATOM] =
+    XInternAtom( _xklDpy, _XKB_RF_NAMES_PROP_ATOM, False );
+  _xklAtoms[XKB_RF_NAMES_PROP_ATOM_BACKUP] =
+    XInternAtom( _xklDpy, "_XKB_RULES_NAMES_BACKUP", False );
 
   _xklAllowSecondaryGroupOnce = False;
+  _xklSkipOneRestore = False;
+  _xklDefaultGroup = -1;
   _xklSecondaryGroupsMask = 0L;
+  _xklPrevAppWindow = 0;
 
   return _XklLoadAllInfo(  )? 0 : _xklLastErrorCode;
 }
@@ -190,7 +217,8 @@ int XklResumeListen(  )
           XkbNamesNotifyMask| \
           XkbControlsNotifyMask| \
           XkbIndicatorStateNotifyMask| \
-          XkbIndicatorMapNotifyMask)
+          XkbIndicatorMapNotifyMask| \
+          XkbNewKeyboardNotifyMask)
 
   XkbSelectEvents( _xklDpy, XkbUseCoreKbd, XKB_EVT_MASK, XKB_EVT_MASK );
 
@@ -210,7 +238,8 @@ int XklResumeListen(  )
                          XkbNamesNotify,
                          XKB_NAMES_EVT_DTL_MASK, XKB_NAMES_EVT_DTL_MASK );
 
-  _XklSelectInputMerging( _xklRootWindow, SubstructureNotifyMask );
+  _XklSelectInputMerging( _xklRootWindow,
+                          SubstructureNotifyMask | PropertyChangeMask );
   _XklGetRealState( &_xklCurState );
   return 0;
 }
@@ -303,50 +332,109 @@ int XklGetRestoreGroup(  )
   {
     return state.group;
   } else
-    XklDebug( 150, "Unbelievable: current client %ld, '%s' has no group\n",
-              _xklCurClient, _XklGetDebugWindowTitle( _xklCurClient ) );
+    XklDebug( 150,
+              "Unbelievable: current client " WINID_FORMAT
+              ", '%s' has no group\n", _xklCurClient,
+              _XklGetDebugWindowTitle( _xklCurClient ) );
   return 0;
 }
 
+void XklSetTransparent( Window win, Bool transparent )
+{
+  Window appWin;
+  Bool wasTransparent;
+  XklDebug( 150, "setting transparent flag %d for " WINID_FORMAT "\n",
+            transparent, win );
+
+  if( !_XklGetAppWindow( win, &appWin ) )
+  {
+    XklDebug( 150, "No app window!\n" );
+    appWin = win;
+//    return;
+  }
+
+  wasTransparent = XklIsTransparent( appWin );
+  XklDebug( 150, "appwin " WINID_FORMAT " was %stransparent\n", appWin,
+            wasTransparent ? "" : "not " );
+  if( transparent && !wasTransparent )
+  {
+    CARD32 prop = 1;
+    XChangeProperty( _xklDpy, appWin, _xklAtoms[XKLAVIER_TRANSPARENT],
+                     XA_INTEGER, 32, PropModeReplace,
+                     ( const unsigned char * ) &prop, 1 );
+  } else if( !transparent && wasTransparent )
+  {
+    XDeleteProperty( _xklDpy, appWin, _xklAtoms[XKLAVIER_TRANSPARENT] );
+  }
+}
+
+Bool XklIsTransparent( Window win )
+{
+  Window appWin;
+
+  if( !_XklGetAppWindow( win, &appWin ) )
+    return False;
+  return _XklIsTransparentAppWindow( appWin );
+
+}
+
 /**
  * "Adds" app window to the set of managed windows.
  * Actually, no data structures involved. The only thing we do is save app state
  * and register ourselves us listeners.
  * Note: User's callback is called
  */
-void _XklAddAppWindow( Window appWin, Window parent, Bool force,
+void _XklAddAppWindow( Window appWin, Window parent, Bool ignoreExistingState,
                        XklState * initState )
 {
   XklState state = *initState;
+  int defGroupToUse = -1;
 
   if( appWin == _xklRootWindow )
     XklDebug( 150, "??? root app win ???\n" );
 
-  XklDebug( 150, "Trying to add window %ld/%s with group %d\n",
+  XklDebug( 150, "Trying to add window " WINID_FORMAT "/%s with group %d\n",
             appWin, _XklGetDebugWindowTitle( appWin ), initState->group );
 
-  if( !force )
+  if( !ignoreExistingState )
   {
     Bool have_state = _XklGetAppState( appWin, &state );
 
     if( have_state )
     {
       XklDebug( 150,
-                "The window %ld does not require to be added, it already has the xklavier state \n",
+                "The window " WINID_FORMAT
+                " does not require to be added, it already has the xklavier state \n",
                 appWin );
       return;
     }
   }
 
+  if( winCallback != NULL )
+    defGroupToUse = ( *winCallback ) ( appWin, parent, winCallbackData );
+
+  if( defGroupToUse == -1 )
+    defGroupToUse = _xklDefaultGroup;
+
+  if( defGroupToUse != -1 )
+    state.group = defGroupToUse;
+
   _XklSaveAppState( appWin, &state );
   _XklSelectInputMerging( appWin, FocusChangeMask | PropertyChangeMask );
 
+  if( defGroupToUse != -1 )
+  {
+    if( _xklCurClient == appWin )
+    {
+      if( ( _xklSecondaryGroupsMask & ( 1 << defGroupToUse ) ) != 0 )
+        XklAllowOneSwitchToSecondaryGroup(  );
+      XklLockGroup( defGroupToUse );
+    }
+  }
+
   if( parent == ( Window ) NULL )
     parent = _XklGetRegisteredParent( appWin );
 
-  if( winCallback != NULL )
-    ( *winCallback ) ( appWin, parent, winCallbackData );
-
   XklDebug( 150, "done\n" );
 }
 
@@ -357,7 +445,6 @@ Bool _XklGetAppWindowBottomToTop( Window win, Window * appWin_return )
 {
   Window parent = ( Window ) NULL, rwin = ( Window ) NULL, *children = NULL;
   int num = 0;
-  Bool rv;
 
   if( win == ( Window ) NULL || win == _xklRootWindow )
   {
@@ -384,8 +471,7 @@ Bool _XklGetAppWindowBottomToTop( Window win, Window * appWin_return )
   if( children != NULL )
     XFree( children );
 
-  rv = _XklGetAppWindowBottomToTop( parent, appWin_return );
-  return rv;
+  return _XklGetAppWindowBottomToTop( parent, appWin_return );
 }
 
 /**
@@ -406,8 +492,9 @@ Bool _XklGetAppWindow( Window win, Window * appWin_return )
   {
     *appWin_return = ( Window ) NULL;
     _xklLastErrorMsg = "The window is either 0 or root";
-    XklDebug( 160,
-              "Window %ld is either 0 or root so could not get the app window for it\n",
+    XklDebug( 150,
+              "Window " WINID_FORMAT
+              " is either 0 or root so could not get the app window for it\n",
               win );
     return False;
   }
@@ -424,9 +511,9 @@ Bool _XklGetAppWindow( Window win, Window * appWin_return )
   if( _xklLastErrorCode != Success )
   {
     *appWin_return = ( Window ) NULL;
-    XklDebug( 160,
-              "Could not get tree for window %ld so could not get the app window for it\n",
-              win );
+    XklDebug( 150,
+              "Could not get tree for window " WINID_FORMAT
+              " so could not get the app window for it\n", win );
     return False;
   }
 
@@ -454,7 +541,7 @@ Bool _XklGetAppWindow( Window win, Window * appWin_return )
   rv = _XklGetAppWindowBottomToTop( parent, appWin_return );
 
   if( !rv )
-    XklDebug( 200, "Could not get the app window for %ld/%s\n",
+    XklDebug( 200, "Could not get the app window for " WINID_FORMAT "/%s\n",
               win, _XklGetDebugWindowTitle( win ) );
 
   return rv;
@@ -473,15 +560,17 @@ Bool _XklLoadWindowTree(  )
 
   XGetInputFocus( _xklDpy, &focused, &revert );
 
-  XklDebug( 160, "initially focused: %ld, '%s'\n", focused );
+  XklDebug( 160, "initially focused: " WINID_FORMAT ", '%s'\n", focused );
 
   haveAppWindow = _XklGetAppWindow( focused, &_xklCurClient );
 
   if( haveAppWindow )
   {
     Bool haveState = _XklGetAppState( _xklCurClient, &_xklCurState );
-    XklDebug( 160, "initial _xklCurClient: %ld, '%s' %s state %d/%X\n",
-              _xklCurClient, _XklGetDebugWindowTitle( _xklCurClient ),
+    XklDebug( 160,
+              "initial _xklCurClient: " WINID_FORMAT
+              ", '%s' %s state %d/%X\n", _xklCurClient,
+              _XklGetDebugWindowTitle( _xklCurClient ),
               ( haveState ? "with" : "without" ),
               ( haveState ? _xklCurState.group : -1 ),
               ( haveState ? _xklCurState.indicators : -1 ) );
@@ -621,7 +710,7 @@ Bool _XklGetAppState( Window appWin, XklState * state_return )
   Atom type_ret;
   int format_ret;
   unsigned long nitems, rest;
-  CARD32 *prop;
+  CARD32 *prop = NULL;
   Bool ret = False;
 
   int grp = -1;
@@ -651,11 +740,13 @@ Bool _XklGetAppState( Window appWin, XklState * state_return )
   }
 
   if( ret )
-    XklDebug( 150, "Appwin %ld, '%s' has the group %d, indicators %X\n",
-              appWin, _XklGetDebugWindowTitle( appWin ), grp, inds );
+    XklDebug( 150,
+              "Appwin " WINID_FORMAT
+              ", '%s' has the group %d, indicators %X\n", appWin,
+              _XklGetDebugWindowTitle( appWin ), grp, inds );
   else
-    XklDebug( 150, "Appwin %ld, '%s' does not have state\n", appWin,
-              _XklGetDebugWindowTitle( appWin ) );
+    XklDebug( 150, "Appwin " WINID_FORMAT ", '%s' does not have state\n",
+              appWin, _XklGetDebugWindowTitle( appWin ) );
 
   return ret;
 }
@@ -682,7 +773,8 @@ void _XklSaveAppState( Window appWin, XklState * state )
                    32, PropModeReplace, ( const unsigned char * ) prop,
                    PROP_LENGTH );
 
-  XklDebug( 160, "Saved the group %d, indicators %X for appwin %ld\n",
+  XklDebug( 160,
+            "Saved the group %d, indicators %X for appwin " WINID_FORMAT "\n",
             state->group, state->indicators, appWin );
 }
 
@@ -692,7 +784,9 @@ void _XklSaveAppState( Window appWin, XklState * state )
 void _XklSelectInput( Window win, long mask )
 {
   if( _xklRootWindow == win )
-    XklDebug( 160, "Someone is looking for %lX on root window ***\n", mask );
+    XklDebug( 160,
+              "Someone is looking for " WINID_FORMAT " on root window ***\n",
+              mask );
 
   XSelectInput( _xklDpy, win, mask );
 }
@@ -716,6 +810,11 @@ void _XklTryCallStateCallback( XklStateChange changeType,
   int group = _xklCurState.group;
   Bool restore = oldState->group == group;
 
+  XklDebug( 150,
+            "changeType: %d, group: %d, secondaryGroupMask: %X, allowsecondary: %d\n",
+            changeType, group, _xklSecondaryGroupsMask,
+            _xklAllowSecondaryGroupOnce );
+
   if( changeType == GROUP_CHANGED )
   {
     if( !restore )
@@ -738,3 +837,23 @@ void _XklTryCallStateCallback( XklStateChange changeType,
                          restore, stateCallbackData );
   }
 }
+
+Bool _XklIsTransparentAppWindow( Window appWin )
+{
+  Atom type_ret;
+  int format_ret;
+  unsigned long nitems, rest;
+  CARD32 *prop = NULL;
+  Bool retVal;
+  if( ( XGetWindowProperty
+        ( _xklDpy, appWin, _xklAtoms[XKLAVIER_TRANSPARENT], 0L, 1, False,
+          XA_INTEGER, &type_ret, &format_ret, &nitems, &rest,
+          ( unsigned char ** ) &prop ) == Success )
+      && ( type_ret == XA_INTEGER ) && ( format_ret == 32 ) )
+  {
+    if( prop != NULL )
+      XFree( prop );
+    return True;
+  }
+  return False;
+}
