@@ -82,6 +82,8 @@ extern void XklDumpXkbDesc( const char *filename, XkbDescPtr kbd );
 
 extern const char *_XklGetEventName( int type );
 
+extern Bool _XklIsTransparentAppWindow( Window appWin );
+
 extern Display *_xklDpy;
 
 extern Bool _xklXkbExtPresent;
@@ -111,6 +113,10 @@ extern int _xklXkbEventType, _xklXkbError;
 #define WM_NAME 0
 #define WM_STATE 1
 #define XKLAVIER_STATE 2
+#define XKLAVIER_TRANSPARENT 3
+#define XKB_RF_NAMES_PROP_ATOM 4
+#define XKB_RF_NAMES_PROP_ATOM_BACKUP 5
+#define TOTAL_ATOMS 6
 
 extern Atom _xklAtoms[];
 
@@ -118,8 +124,16 @@ extern XkbRF_VarDefsRec _xklVarDefs;
 
 extern Bool _xklAllowSecondaryGroupOnce;
 
+extern int _xklDefaultGroup;
+
+extern Bool _xklSkipOneRestore;
+
 extern int _xklSecondaryGroupMask;
 
 extern int _xklDebugLevel;
 
+extern Window _xklPrevAppWindow;
+
+#define WINID_FORMAT "%lx"
+
 #endif
